using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.SqlClient;

namespace Akadia.AutoIncrement
{
	public class AutoIncrement : System.Windows.Forms.Form
	{
		private System.Windows.Forms.DataGrid dgShowCat;
		private System.Windows.Forms.Label lblUpdate;
		private System.Windows.Forms.Button btnUpdate;
		private System.ComponentModel.Container components = null;

		private DataSet catDS;
		private SqlDataAdapter catDA;
		private System.Windows.Forms.TextBox txtCatName;

		public AutoIncrement()
		{
			// Initialize GUI
			InitializeComponent();

			// Fetch Categories Table into DataSet
			SqlConnection nwindConn = new SqlConnection("Data Source=XEON; " +
				"Initial Catalog=Northwind; User id=sa; Password=manager;");

			catDA = new SqlDataAdapter("SELECT CategoryID, "+
				"CategoryName FROM Categories ORDER BY CategoryID", nwindConn);

			// Change the INSERT command text to include a SELECT
			// command as well, and make sure the INSERT command's
			// UpdatedRowSource is set to UpdateRowSource.FirstReturnedRecord
			SqlCommand cmdIn = new SqlCommand(
				"INSERT INTO Categories(CategoryName) VALUES (@CategoryName);" +
				"SELECT CategoryID, CategoryName " +
				"  FROM Categories " +
				" WHERE (CategoryID = @@IDENTITY)",nwindConn);

			cmdIn.Parameters.Add("@CategoryName",
				typeof(String)).SourceColumn="CategoryName";
			catDA.InsertCommand = cmdIn;

			// The data in the first returned row is mapped to the
			// changed row in the DataSet.
			cmdIn.UpdatedRowSource = UpdateRowSource.FirstReturnedRecord;

			// Set UpdateRowSource and Register RowUpdatedEventHandler
			catDA.RowUpdated += new SqlRowUpdatedEventHandler(myHandler);
			DataTable catTable = new DataTable("Categories");

			// Fetch Data into Dataset
			nwindConn.Open();
			catDS = new DataSet("CategoriesDS");
			catDA.Fill(catDS, "Categories");
			dgShowCat.SetDataBinding(catDS,"Categories");
			nwindConn.Close();
		}

		// Next, you'll need to select the autoincrement
		// value from the server back into the DataSet upon update.
		// SQL Server provides a couple of ways to do this, including
		// the @@Identity variable that returns the ID of the last row inserted 
		// within your scope. Finally, if you want to continue using the DataSet
		// on the client, you would want to update the DataSet on the client by
		// mergine the DataSet updated with new values from the server. In order
		// to do this, you need to make sure that the DataAdapter does not accept
		// changes when it updates the rows in the DataSet; otherwise the inserted
		// row that has been updated with the new values will lose the
		// original primary key values, and when you merge with the original
		// dataset it will be added as a new row. In order to prevent the
		// DataAdapter from calling AcceptChanges, you'll need to intercept
		// the OnRowChanged event and tell the DataAdapter to skip calling
		// AcceptChanges on the current row.
		public static void myHandler(object adapter, SqlRowUpdatedEventArgs e) 
		{
			// Don't call AcceptChanges
			e.Status = UpdateStatus.SkipCurrentRow;
		}			
			
		// Update Dataset using the INSERT Command and fetch
		// Autoincrement Value back to the Dataset
		private void btnUpdate_Click(object sender, System.EventArgs e)
		{
			// When working with auto-incrementing columns at a data source
			// and auto-incrementing columns in a DataSet, you create the column
			// in the DataSet with an AutoIncrementStep of -1 and an 
			// AutoIncrementSeed of 0, as well as ensuring that your data source
			// generates auto-incrementing identity values starting from 1 and
			// incrementing with a positive step value. As a result, the DataSet
			// generates negative numbers for auto-incremented values that do not
			// conflict with the positive auto-increment values generated by
			// the data source.
			catDS.Tables["Categories"].Columns["CategoryID"].AutoIncrementStep = -1;
			catDS.Tables["Categories"].Columns["CategoryID"].AutoIncrementSeed = 0;

			// Create a new Row and UPDATE it to the database
			DataRow newRow = catDS.Tables["Categories"].NewRow();
			newRow["CategoryName"] = txtCatName.Text;
			catDS.Tables["Categories"].Rows.Add(newRow);			
			catDA.Update(catDS, "Categories");	
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.dgShowCat = new System.Windows.Forms.DataGrid();
			this.txtCatName = new System.Windows.Forms.TextBox();
			this.lblUpdate = new System.Windows.Forms.Label();
			this.btnUpdate = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.dgShowCat)).BeginInit();
			this.SuspendLayout();
			// 
			// dgShowCat
			// 
			this.dgShowCat.CaptionText = "Table Categories";
			this.dgShowCat.DataMember = "";
			this.dgShowCat.HeaderForeColor = System.Drawing.SystemColors.ControlText;
			this.dgShowCat.Location = new System.Drawing.Point(6, 5);
			this.dgShowCat.Name = "dgShowCat";
			this.dgShowCat.Size = new System.Drawing.Size(393, 200);
			this.dgShowCat.TabIndex = 0;
			// 
			// txtCatName
			// 
			this.txtCatName.Location = new System.Drawing.Point(90, 221);
			this.txtCatName.Name = "txtCatName";
			this.txtCatName.Size = new System.Drawing.Size(133, 20);
			this.txtCatName.TabIndex = 1;
			this.txtCatName.Text = "";
			// 
			// lblUpdate
			// 
			this.lblUpdate.Location = new System.Drawing.Point(0, 223);
			this.lblUpdate.Name = "lblUpdate";
			this.lblUpdate.RightToLeft = System.Windows.Forms.RightToLeft.Yes;
			this.lblUpdate.Size = new System.Drawing.Size(83, 15);
			this.lblUpdate.TabIndex = 2;
			this.lblUpdate.Text = "Category Name";
			// 
			// btnUpdate
			// 
			this.btnUpdate.Location = new System.Drawing.Point(230, 221);
			this.btnUpdate.Name = "btnUpdate";
			this.btnUpdate.Size = new System.Drawing.Size(75, 21);
			this.btnUpdate.TabIndex = 3;
			this.btnUpdate.Text = "Update";
			this.btnUpdate.Click += new System.EventHandler(this.btnUpdate_Click);
			// 
			// AutoIncrement
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(404, 255);
			this.Controls.Add(this.btnUpdate);
			this.Controls.Add(this.lblUpdate);
			this.Controls.Add(this.txtCatName);
			this.Controls.Add(this.dgShowCat);
			this.Name = "AutoIncrement";
			this.Text = "Return Autoincrement Primary Key in Data Set";
			((System.ComponentModel.ISupportInitialize)(this.dgShowCat)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new AutoIncrement());
		}
	}
}
